package nl.iobyte.themepark.api;

import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.attraction.Region;
import nl.iobyte.themepark.api.events.attraction.CreateAttractionEvent;
import nl.iobyte.themepark.api.events.attraction.RemoveAttractionEvent;
import nl.iobyte.themepark.api.events.region.CreateRegionEvent;
import nl.iobyte.themepark.api.events.region.RemoveRegionEvent;
import nl.iobyte.themepark.api.ridecount.CountManager;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import java.util.LinkedHashMap;

public class API {

    private static LinkedHashMap<String, Region> regions = new LinkedHashMap<>();
    private static LinkedHashMap<String, Attraction> attractions = new LinkedHashMap<>();

    public static void addRegion(Region region) {
        if(isRegion(region.getId()))
            return;

        regions.put(region.getId(), region);

        CreateRegionEvent e = new CreateRegionEvent(region);
        if(Bukkit.isPrimaryThread()) {
            Bukkit.getPluginManager().callEvent(e);
        } else {
            Bukkit.getScheduler().runTask(ThemeParkPlugin.getInstance(), () -> Bukkit.getPluginManager().callEvent(e));
        }
    }

    public static boolean isRegion(String id) {
        if(id == null || id.isEmpty())
            return false;

        return regions.containsKey(id);
    }

    public static Region getRegion(String id) {
        if(id == null || id.isEmpty())
            return null;

        return regions.get(id);
    }

    public static LinkedHashMap<String, Region> getRegions() {
        return regions;
    }

    public static void removeRegion(String id) {
        Region region = getRegion(id);
        if(region == null)
            return;

        for(Attraction attraction : region.getAttractions().values())
            attractions.remove(attraction.getId());

        regions.remove(id);

        RemoveRegionEvent e = new RemoveRegionEvent(region);
        if(Bukkit.isPrimaryThread()) {
            Bukkit.getPluginManager().callEvent(e);
        } else {
            Bukkit.getScheduler().runTask(ThemeParkPlugin.getInstance(), () -> Bukkit.getPluginManager().callEvent(e));
        }
    }

    public static void addAttraction(Attraction attraction) {
        if(!isRegion(attraction.getRegionId()))
            return;

        if(isAttraction(attraction.getId()))
            return;

        getRegion(attraction.getRegionId()).addAttraction(attraction);
        attractions.put(attraction.getId(), attraction);
        CountManager.addCounter(attraction);

        CreateAttractionEvent e = new CreateAttractionEvent(attraction);
        if(Bukkit.isPrimaryThread()) {
            Bukkit.getPluginManager().callEvent(e);
        } else {
            Bukkit.getScheduler().runTask(ThemeParkPlugin.getInstance(), () -> Bukkit.getPluginManager().callEvent(e));
        }
    }

    public static boolean isAttraction(String id) {
        if(id == null || id.isEmpty())
            return false;

        return attractions.containsKey(id);
    }

    public static Attraction getAttraction(String id) {
        if(id == null || id.isEmpty())
            return null;

        return attractions.get(id);
    }

    public static Attraction getAttractionFromName(String name) {
        if(name == null || name.isEmpty())
            return null;

        for(Attraction attraction : attractions.values()) {
            if(!color(attraction.getName()).equals(name))
                continue;

            return attraction;
        }

        return null;
    }

    public static LinkedHashMap<String, Attraction> getAttractions() {
        return attractions;
    }

    public static void removeAttraction(String id) {
        if(!isAttraction(id))
            return;

        Attraction attraction = getAttraction(id);
        if(isRegion(attraction.getRegionId()))
            getRegion(attraction.getRegionId()).removeAttraction(attraction);

        attractions.remove(id);
        CountManager.removeCounter(id);

        RemoveAttractionEvent e = new RemoveAttractionEvent(attraction);
        if(Bukkit.isPrimaryThread()) {
            Bukkit.getPluginManager().callEvent(e);
        } else {
            Bukkit.getScheduler().runTask(ThemeParkPlugin.getInstance(), () -> Bukkit.getPluginManager().callEvent(e));
        }
    }

    private static String color(String string) {
        return ChatColor.translateAlternateColorCodes('&', string);
    }

}
