package nl.iobyte.themepark.command;

import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.API;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.attraction.Region;
import nl.iobyte.themepark.api.attraction.component.Status;
import nl.iobyte.themepark.api.attraction.component.Type;
import nl.iobyte.themepark.api.attraction.manager.StatusManager;
import nl.iobyte.themepark.api.utils.ItemBuilder;
import nl.iobyte.themepark.config.YamlConfig;
import nl.iobyte.themepark.menu.AttractionLoader;
import nl.iobyte.themepark.menu.AttractionMenu;
import nl.iobyte.themepark.util.Color;
import nl.iobyte.themepark.util.MessageUtil;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.command.CommandSender;
import org.bukkit.command.defaults.BukkitCommand;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Minecart;
import org.bukkit.entity.Player;
import java.util.ArrayList;
import java.util.UUID;

public class MainCommand extends BukkitCommand {

    private final String name;

    public MainCommand(String name) {
        super(name);

        this.name = name;
    }

    public boolean execute(CommandSender sender,String label, String[] args) {
        if (args.length == 0 || args[0].equalsIgnoreCase("help")) {
            sender.sendMessage(Color.color("&f&L>==== &r&6ThemePark &L&f ====<"));
            sender.sendMessage(Color.color("&f/" + name + " help"));
            sender.sendMessage(Color.color("&f/" + name + " list"));
            sender.sendMessage(Color.color("&f/" + name + " warp <id>"));
            if(sender.hasPermission("themepark.admin") || sender.hasPermission("themepark.item")) {
                sender.sendMessage(Color.color("&f/" + name + " toggleitem"));
                sender.sendMessage(Color.color("&f/" + name + " getitem"));
            }

            if(sender.hasPermission("themepark.admin")) {
                sender.sendMessage(Color.color("&f/" + name + " region create <id> <name>"));
                sender.sendMessage(Color.color("&f/" + name + " region name <id> <name>"));
                sender.sendMessage(Color.color("&f/" + name + " region remove <id>"));
                sender.sendMessage(Color.color("&f/" + name + " attraction create <region> <id> <RIDE/SHOW/GLOBAL> <name>"));
                sender.sendMessage(Color.color("&f/" + name + " attraction name <id> <name>"));
                sender.sendMessage(Color.color("&f/" + name + " attraction status <id> <status>"));
                sender.sendMessage(Color.color("&f/" + name + " attraction location <id>"));
                sender.sendMessage(Color.color("&f/" + name + " attraction remove <id>"));
            }

            return true;
        }

        if (args[0].equalsIgnoreCase("list")) {
            sender.sendMessage(Color.color("&f&L>==== &r&6ThemePark &f&L ====<"));
            if (API.getAttractions().size() == 0) {
                sender.sendMessage(Color.color(MessageUtil.getMessage("attraction.notfound")));
                return true;
            }

            for (Attraction attraction : API.getAttractions().values()) {
                String message = MessageUtil.getMessage("attraction.list");
                message = message.replace("{id}", attraction.getId());
                message = message.replace("{name}", attraction.getName());
                message = message.replace("{region}", attraction.getRegion().getName());
                message = message.replace("{type}", attraction.getType().toString());
                message = message.replace("{status}", StatusManager.getName(attraction.getStatus()));
                sender.sendMessage(Color.color(message));
            }

            return true;
        }

        if(args[0].equalsIgnoreCase("warp")) {
            if(args.length < 2) {
                sender.sendMessage(Color.color( MessageUtil.getMessage("prefix")+" &4/" + name + " warp <id>"));
                return true;
            }

            String id = args[1];
            if(!API.isAttraction(id)) {
                String message = MessageUtil.getMessage("attraction.not");
                message = message.replace("{id}", id);
                sender.sendMessage(Color.color(message));
                return true;
            }

            if(sender instanceof Player) {
                Attraction attraction = API.getAttraction(id);
                Player player = (Player) sender;
                if(!StatusManager.canTeleport(attraction.getStatus())) {
                    String message = MessageUtil.getMessage("attraction.teleport.status");
                    message = message.replace("{name}", attraction.getName());
                    message = message.replace("{status}", StatusManager.getName(attraction.getStatus()));
                    player.sendMessage(Color.color(message));
                    return true;
                }

                if(attraction.getLocation() == null)
                    return true;

                player.teleport(attraction.getLocation());
                if(player.isInsideVehicle())
                    if(player.getVehicle() instanceof Minecart)
                        return true;

                player.sendMessage(Color.color(MessageUtil.getMessage("attraction.teleport.success").replace("{name}", attraction.getName())));
                return true;
            } else {
                sender.sendMessage(Color.color(MessageUtil.getMessage("onlyplayers")));
                return true;
            }
        }

        if(!sender.hasPermission("themepark.admin") && !sender.hasPermission("themepark.item")) {
            sender.sendMessage(Color.color(MessageUtil.getMessage("nopermission")));
            return true;
        }

        if(args[0].equalsIgnoreCase("toggleitem")) {
            if(!(sender instanceof Player)) {
                sender.sendMessage(Color.color(MessageUtil.getMessage("onlyplayers")));
                return true;
            }

            Player player = (Player) sender;
            UUID uuid = player.getUniqueId();

            YamlConfig config = ThemeParkPlugin.getInstance().getData();
            boolean b = config.getConfig().getBoolean(uuid.toString() + ".item");
            b = !b;

            config.getConfig().set(uuid.toString() + ".item", b);
            config.save();

            String message = MessageUtil.getMessage("menu-item.toggle");
            message = message.replace("{status}", b ? MessageUtil.getMessage("menu-item.enabled") : MessageUtil.getMessage("menu-item.disabled"));
            sender.sendMessage(Color.color(message));
            return true;
        }

        if(args[0].equalsIgnoreCase("getitem")) {
            YamlConfig settings = ThemeParkPlugin.getInstance().getSettings();
            Material material = Material.getMaterial(settings.getConfig().getString("item.material"));
            short data = Short.parseShort(settings.getConfig().getString("item.data"));
            String name = Color.color(settings.getConfig().getString("item.display-name"));
            if(material == null || name.isEmpty())
                return true;

            Player player;
            if(args.length >= 2) {
                String string = args[1];
                if(Bukkit.getPlayerExact(string) == null) {
                    String message = MessageUtil.getMessage("noplayer");
                    message = message.replace("{name}", string);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                player = Bukkit.getPlayerExact(string);
            } else {
                if(!(sender instanceof Player)) {
                    sender.sendMessage(Color.color(MessageUtil.getMessage("onlyplayers")));
                    return true;
                }

                player = (Player) sender;
            }


            ItemBuilder builder = new ItemBuilder(material, 1, data);
            builder.setName(name);
            player.getInventory().addItem(builder.getItem());
        }

        if(!sender.hasPermission("themepark.admin")) {
            sender.sendMessage(Color.color(MessageUtil.getMessage("nopermission")));
            return true;
        }

        if (args[0].equalsIgnoreCase("region")) {
            if(args.length < 2)
                return false;

            if (args[1].equalsIgnoreCase("create")) {
                if (args.length < 4 ) {
                    sender.sendMessage(Color.color(MessageUtil.getMessage("prefix")+" &4/" + name + " region create <id> <name>"));
                    return true;
                }

                String id = args[2];
                if (API.isRegion(id)) {
                    String message = MessageUtil.getMessage("region.exists");
                    message = message.replace("{id}", id);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                StringBuilder builder = new StringBuilder();
                builder.append(args[3]);
                for(int i = 4; i < args.length; i++)
                    builder.append(" ").append(args[i]);

                String name = builder.toString();
                Region region = new Region(id, name, new ArrayList<>());
                API.addRegion(region);
                AttractionMenu.load();
                AttractionLoader.addRegion(id);

                YamlConfig config = ThemeParkPlugin.getInstance().getRegions();
                FileConfiguration fc = config.getConfig();
                String path = "regions."+id+".";
                fc.set(path+"name", name);
                fc.set(path+"lore", region.getLore());
                config.save();

                sender.sendMessage(Color.color(MessageUtil.getMessage("region.create").replace("{name}", name)));
                return true;
            }

            if(args[1].equalsIgnoreCase("name")) {
                if (args.length < 4 ) {
                    sender.sendMessage(Color.color("&f/" + name + " region name <id> <name>"));
                    return true;
                }

                String id = args[2];
                if(!API.isRegion(id)) {
                    String message = MessageUtil.getMessage("region.not");
                    message = message.replace("{id}", id);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                StringBuilder builder = new StringBuilder();
                builder.append(args[3]);
                for(int i = 4; i < args.length; i++)
                    builder.append(" ").append(args[i]);

                String name = builder.toString();
                Region region = API.getRegion(id);
                region.setName(name);
                sender.sendMessage(Color.color(MessageUtil.getMessage("region.name").replace("{name}", region.getName())));
                return true;
            }

            if(args[1].equalsIgnoreCase("remove")) {
                if (args.length < 3 ) {
                    sender.sendMessage(Color.color("&f/" + name + " region remove <id>"));
                    return true;
                }

                String id = args[2];
                if(!API.isRegion(id)) {
                    String message = MessageUtil.getMessage("region.not");
                    message = message.replace("{id}", id);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                Region region = API.getRegion(id);
                API.removeRegion(id);
                sender.sendMessage(Color.color(MessageUtil.getMessage("region.remove").replace("{name}", region.getName())));
                return true;
            }
        }

        if (args[0].equalsIgnoreCase("attraction")) {
            if(args.length < 2)
                return false;

            if (args[1].equalsIgnoreCase("create")) {
                if (args.length < 6 ) {
                    sender.sendMessage(Color.color(MessageUtil.getMessage("prefix")+" &4/" + name + " attraction create <region> <id> <RIDE/SHOW/GLOBAL> <name>"));
                    return true;
                }

                String id = args[2];
                if(!API.isRegion(id)) {
                    String message = MessageUtil.getMessage("region.not");
                    message = message.replace("{id}", id);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                id = args[3];
                if (API.isAttraction(id)) {
                    String message = MessageUtil.getMessage("attraction.exists");
                    message = message.replace("{id}", id);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                Type type = Type.getType(args[4]);
                if(type == null) {
                    String message = MessageUtil.getMessage("no-type");
                    message = message.replace("{type}", args[4]);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                StringBuilder builder = new StringBuilder();
                builder.append(args[5]);
                for(int i = 6; i < args.length; i++)
                    builder.append(" ").append(args[i]);

                String name = builder.toString();
                Attraction attraction = new Attraction(args[3], name, args[2], null, type, type.getDefault());
                YamlConfig config = AttractionLoader.getConfig(attraction.getRegionId());
                if(config == null) {
                    String prefix = MessageUtil.getMessage("prefix");
                    String message = prefix+ " &4There was a problem when trying to save the attraction.";
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                FileConfiguration fc = config.getConfig();
                String path = "attractions."+attraction.getId()+".";
                fc.set(path+"name", attraction.getName());
                fc.set(path+"type", type.toString());
                fc.set(path+"status", attraction.getStatus().toString());
                config.save();

                API.addAttraction(attraction);
                AttractionMenu.load();
                sender.sendMessage(Color.color(MessageUtil.getMessage("attraction.create").replace("{name}", name)));
                return true;
            }

            if(args[1].equalsIgnoreCase("name")) {
                if (args.length < 4 ) {
                    sender.sendMessage(Color.color("&f/" + name + " attraction name <id> <name>"));
                    return true;
                }

                String id = args[2];
                if(!API.isAttraction(id)) {
                    String message = MessageUtil.getMessage("attraction.not");
                    message = message.replace("{id}", id);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                StringBuilder builder = new StringBuilder();
                builder.append(args[3]);
                for(int i = 4; i < args.length; i++)
                    builder.append(" ").append(args[i]);

                String name = builder.toString();
                Attraction attraction = API.getAttraction(id);
                attraction.setName(name);
                sender.sendMessage(Color.color(MessageUtil.getMessage("attraction.name").replace("{name}", attraction.getName())));
                return true;
            }

            if (args[1].equalsIgnoreCase("status")) {
                if (args.length < 4 ) {
                    sender.sendMessage(Color.color(MessageUtil.getMessage("prefix")+" &4/" + name + " attraction status <id> <status>"));
                    return true;
                }

                String id = args[2];
                if (!API.isAttraction(id)) {
                    String message = MessageUtil.getMessage("attraction.not");
                    message = message.replace("{id}", id);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                Status status = Status.getStatus(args[3].toUpperCase());
                if (status == null) {
                    String message = MessageUtil.getMessage("no-status");
                    message = message.replace("{status}", args[3]);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                Attraction attraction = API.getAttraction(id);
                if(!attraction.getType().containsStatus(status)) {
                    String message = MessageUtil.getMessage("attraction.nostatus");
                    message = message.replace("{name}", attraction.getName());
                    message = message.replace("{status}", status.toString());
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                if(attraction.getStatus() == status)
                    return true;

                Player player = (sender instanceof Player) ? (Player) sender : null;
                attraction.setStatus(status, player);
                return true;
            }

            if(args[1].equalsIgnoreCase("location")) {
                if (args.length < 3 ) {
                    sender.sendMessage(Color.color("&f/" + name + " attraction location <id>"));
                    return true;
                }

                String id = args[2];
                if(!API.isAttraction(id)) {
                    String message = MessageUtil.getMessage("attraction.not");
                    message = message.replace("{id}", id);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                if(!(sender instanceof Player)) {
                    sender.sendMessage(Color.color(MessageUtil.getMessage("onlyplayers")));
                    return true;
                }

                Attraction attraction = API.getAttraction(id);
                Location location = ((Player) sender).getLocation();
                attraction.setLocation(location.clone());
                sender.sendMessage(Color.color(MessageUtil.getMessage("attraction.location").replace("{name}", attraction.getName())));
                return true;
            }

            if(args[1].equalsIgnoreCase("remove")) {
                if (args.length < 3 ) {
                    sender.sendMessage(Color.color("&f/" + name + " attraction remove <id>"));
                    return true;
                }

                String id = args[2];
                if(!API.isAttraction(id)) {
                    String message = MessageUtil.getMessage("attraction.not");
                    message = message.replace("{id}", id);
                    sender.sendMessage(Color.color(message));
                    return true;
                }

                Attraction attraction = API.getAttraction(id);
                API.removeAttraction(id);
                sender.sendMessage(Color.color(MessageUtil.getMessage("attraction.remove").replace("{name}", attraction.getName())));
                return true;
            }
        }

        return false;
    }

}