package nl.iobyte.themepark.command;

import nl.iobyte.menuapi.map.WeakConcurrentHashMap;
import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.API;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.events.ridecount.ChangeCountEvent;
import nl.iobyte.themepark.api.events.ridecount.PreProcessCountEvent;
import nl.iobyte.themepark.api.ridecount.AttractionCount;
import nl.iobyte.themepark.api.ridecount.CountManager;
import nl.iobyte.themepark.ridecount.RideCountAPI;
import nl.iobyte.themepark.util.Color;
import nl.iobyte.themepark.util.MessageUtil;
import nl.iobyte.themepark.util.SpigotPlayerSelector;
import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.UUID;
import java.util.concurrent.atomic.AtomicBoolean;

public class RidecountCommand implements CommandExecutor {

    private static final WeakConcurrentHashMap<String, String> data = new WeakConcurrentHashMap<>(5 * 60 * 1000);

    @Override
    public boolean onCommand(CommandSender sender, Command cmd, String label, String[] args) {
        if(!cmd.getName().equalsIgnoreCase("ridecount"))
            return false;

        if(!sender.hasPermission("themepark.admin")) {
            sender.sendMessage(Color.color(MessageUtil.getMessage("nopermission")));
            return true;
        }

        if(args.length == 0 || args[0].equalsIgnoreCase("help")) {
            sender.sendMessage(Color.color("&f&L>==== &r&6Ridecount &L&f ====<"));
            sender.sendMessage(Color.color("&f/ridecount help"));
            sender.sendMessage(Color.color("&f/ridecount get <player> <id>"));
            sender.sendMessage(Color.color("&f/ridecount add <player> <id> <amount>"));
            return true;
        }

        if(args[0].equalsIgnoreCase("get")) {
            if(args.length < 3) {
                sender.sendMessage(Color.color(MessageUtil.getMessage("prefix")+" &4/ridecount get <player> <id>"));
                return true;
            }

            if(!API.isAttraction(args[2])) {
                String message = MessageUtil.getMessage("attraction.not");
                message = message.replace("{id}", args[2]);
                sender.sendMessage(Color.color(message));
                return true;
            }

            Attraction attraction = API.getAttraction(args[2]);
            RideCountAPI.getCount(args[1], attraction).thenAccept(points -> {
                if(points < 0) {
                    sender.sendMessage(Color.color(MessageUtil.getMessage("noplayer").replace("{name}", args[1])));
                    return;
                }

                String mesage = MessageUtil.getMessage("ridecount.count");
                mesage = mesage.replace("{name}", attraction.getName());
                mesage = mesage.replace("{count}", points.toString());
                sender.sendMessage(Color.color(mesage));
            });
            return true;
        }

        if(args[0].equalsIgnoreCase("add")) {
            if(args.length < 4) {
                sender.sendMessage(Color.color(MessageUtil.getMessage("prefix")+" &4/ridecount add <player> <id> <amount>"));
                return true;
            }

            if(!API.isAttraction(args[2])) {
                String message = MessageUtil.getMessage("attraction.not");
                message = message.replace("{id}", args[2]);
                sender.sendMessage(Color.color(message));
                return true;
            }

            if(!isInteger(args[3])) {
                sender.sendMessage(Color.color(MessageUtil.getMessage("nonumber").replace("{number}", args[3])));
                return true;
            }

            int amount = Integer.parseInt(args[3]);
            Attraction attraction = API.getAttraction(args[2]);

            AtomicBoolean b = new AtomicBoolean(false);
            List<UUID> ids = executeArg(sender, args[1]);

            AttractionCount count = CountManager.getCounter(attraction.getId());
            for(UUID uuid : ids) {
                PreProcessCountEvent event = new PreProcessCountEvent(uuid, attraction, amount);
                Bukkit.getPluginManager().callEvent(event);

                int u_amount = event.getCount();
                RideCountAPI.getCount(uuid, attraction).thenAccept(before -> RideCountAPI.addCount(uuid, attraction, u_amount).thenAccept(result -> {
                    if (!result)
                        return;

                    if(count == null || !count.hasCount(uuid)) {
                        Bukkit.getScheduler().runTask(ThemeParkPlugin.getInstance(), () -> {
                            ChangeCountEvent e = new ChangeCountEvent(attraction, count, uuid, before, before + u_amount);
                            Bukkit.getPluginManager().callEvent(e);
                        });
                    }

                    if(b.get())
                        return;

                    b.set(true);
                    String mesage = MessageUtil.getMessage("ridecount.add");
                    mesage = mesage.replace("{name}", API.getAttraction(args[2]).getName());
                    mesage = mesage.replace("{count}", Integer.toString(u_amount));
                    sender.sendMessage(Color.color(mesage));
                }));
            }
            return true;
        }

        return false;
    }

    private List<UUID> executeArg(CommandSender sender, String arg) {
        List<UUID> array = new ArrayList<>();
        for(Player p : new SpigotPlayerSelector(arg).getPlayers(sender))
            array.add(p.getUniqueId());

        return array;
    }

    private boolean isInteger(String s) {
        int radix = 10;
        Scanner sc = new Scanner(s.trim());
        if(!sc.hasNextInt(radix)) return false;
        sc.nextInt(radix);
        return !sc.hasNext();
    }

}

