package nl.iobyte.themepark.database;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import nl.iobyte.themepark.ThemeParkPlugin;
import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;
import java.util.Properties;

public class SQLite extends DB {

    private String dbName, v;
    private HikariDataSource source;

    /**
     * Initialize a new connection
     *
     * @param dbName The database name
     */
    public SQLite(String dbName) {
        this.dbName = dbName;

        File dbFile = new File(ThemeParkPlugin.getInstance().getDataFolder(), dbName + ".db");
        if (!dbFile.exists()) {
            try {
                System.out.println("Generating the " + dbName + ".db!");
                if (!dbFile.createNewFile()) {
                    System.out.println("Couldn't generate the " + dbName + ".db!");
                    return;
                }
            } catch (IOException e) {
                System.out.println("Couldn't generate the " + dbName + ".db!");
                return;
            }
        }

        HikariConfig config = new HikariConfig();
        config.setPoolName(ThemeParkPlugin.getInstance().getName());
        config.setUsername(null);
        config.setPassword(null);
        config.setDriverClassName("org.sqlite.JDBC");
        config.setConnectionTestQuery("SELECT 1");
        config.setMaximumPoolSize(5);

        Properties prop = new Properties();
        prop.setProperty("date_string_format", "yyyy-MM-dd HH:mm:ss");

        config.setJdbcUrl("jdbc:sqlite:" + dbFile.getAbsolutePath());
        config.setDataSourceProperties(prop);
        source = new HikariDataSource(config);

        try(Connection conn = getConnection()) {
            PreparedStatement statement = conn.prepareStatement("SELECT sqlite_version() AS version");
            statement.closeOnCompletion();

            ResultSet set = statement.executeQuery();
            while(set.next()) {
                v = set.getString("version");
            }

            set.close();
        } catch(Exception e) {
            e.printStackTrace();
        }

        if(v == null) {
            System.out.println("Unable to get SQLite version.");
        } else {
            System.out.println("SQLite database running on version: "+v);
        }
    }

    public String getVersion() {
        return v;
    }

    /**
     * Get connection
     * @return Connection Instance
     */
    public Connection getConnection() throws SQLException {
        return source.getConnection();
    }

    /**
     * Close connection
     */
    public void closeConnection() {
        System.out.println("Closing the database connection for " + dbName + ".db!");
        source.close();
    }

}