package nl.iobyte.themepark.listener;

import net.md_5.bungee.api.chat.BaseComponent;
import net.md_5.bungee.api.chat.ClickEvent;
import net.md_5.bungee.api.chat.HoverEvent;
import net.md_5.bungee.api.chat.TextComponent;
import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.events.attraction.*;
import nl.iobyte.themepark.api.utils.LocationSerializer;
import nl.iobyte.themepark.config.YamlConfig;
import nl.iobyte.themepark.database.DBManager;
import nl.iobyte.themepark.dynmap.MapMarker;
import nl.iobyte.themepark.menu.AttractionLoader;
import nl.iobyte.themepark.menu.AttractionMenu;
import nl.iobyte.themepark.sign.SignManager;
import nl.iobyte.themepark.util.Color;
import nl.iobyte.themepark.util.MessageUtil;
import nl.iobyte.workchain.components.Work;
import org.bukkit.Bukkit;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;

import java.util.HashMap;

public class AttractionListener implements Listener {

    @EventHandler
    public void onCreate(CreateAttractionEvent e) {
        if(AttractionLoader.loading)
            return;

        AttractionMenu.load();

        Attraction attraction = e.getAttraction();
        String config_path = "attractions." + attraction.getId();
        YamlConfig yc = AttractionLoader.getConfig(attraction.getRegionId());
        FileConfiguration config = yc.getConfig();
        config.set(config_path+".name", attraction.getName());
        config.set(config_path+".type", attraction.getType().toString());
        config.set(config_path+".status", attraction.getStatus().toString());
        config.set(config_path+".location", LocationSerializer.toString(attraction.getLocation()));
        yc.save();

        MapMarker.getMarker().setAttractionMarker(attraction);
        Work.firstTask(() -> DBManager.getDatabase("public")).abortIfNull().lastTask(database -> {
            HashMap<Integer, Object> obj = new HashMap<>();
            obj.put(1, attraction.getId());
            obj.put(2, attraction.getName());
            obj.put(3, attraction.getRegionId());
            obj.put(4, attraction.getType().toString());
            obj.put(5, attraction.getStatus().toString());

            database.execute("INSERT IGNORE INTO attraction(id, name, region_id, type, status) VALUES (?, ?, ?, ?, ?)", obj);
        }).execute();
    }

    @EventHandler
    public void onChange(ChangeStatusEvent e) {
        AttractionMenu.updateAttraction(e.getAttraction());

        Attraction attraction = e.getAttraction();
        YamlConfig config = AttractionLoader.getConfig(attraction.getRegionId());
        config.getConfig().set("attractions." + attraction.getId() + ".status", e.getAfter().toString());
        config.save();

        SignManager.update(attraction);
        Work.firstTask(() -> DBManager.getDatabase("public")).abortIfNull().lastTask(database -> {
            HashMap<Integer, Object> obj = new HashMap<>();
            obj.put(1, e.getAfter().toString());
            obj.put(2, attraction.getId());
            database.execute("UPDATE attraction SET status=? WHERE id=?", obj);
        }).execute();

        String command = ThemeParkPlugin.getInstance().getSettings().getConfig().getString("command");
        if(command == null || command.isEmpty())
            return;

        HashMap<String, String> change = new HashMap<>();
        change.put("{name}", attraction.getName());
        ClickEvent click = new ClickEvent(ClickEvent.Action.RUN_COMMAND, "/"+command+" warp "+attraction.getId());
        HoverEvent hover = new HoverEvent(HoverEvent.Action.SHOW_TEXT, TextComponent.fromLegacyText(Color.color(ThemeParkPlugin.getInstance().getMessages().getConfig().getString("attraction.changed.hover"))));
        if(hover.getValue()[0].toPlainText().isEmpty())
            hover = null;

        BaseComponent message = MessageUtil.parseTP("attraction.changed."+e.getAfter().toString(), change, click, hover);
        if(message == null)
            return;

        for(Player player : Bukkit.getOnlinePlayers())
            player.spigot().sendMessage(message);
    }

    @EventHandler
    public void onChange(ChangeTypeEvent e) {
        AttractionMenu.updateAttraction(e.getAttraction());

        Attraction attraction = e.getAttraction();
        YamlConfig config = AttractionLoader.getConfig(attraction.getRegionId());
        config.getConfig().set("attractions." + attraction.getId() + ".type", e.getAfter().toString());
        config.save();

        SignManager.update(attraction);
        Work.firstTask(() -> DBManager.getDatabase("public")).abortIfNull().lastTask(database -> {
            HashMap<Integer, Object> obj = new HashMap<>();
            obj.put(1, e.getAfter().toString());
            obj.put(2, attraction.getStatus().toString());
            obj.put(3, attraction.getId());

            database.execute("UPDATE attraction SET type=?, status=? WHERE id=?", obj);
        }).execute();
    }

    @EventHandler
    public void onChange(ChangeNameEvent e) {
        AttractionMenu.updateAttraction(e.getAttraction());

        Attraction attraction = e.getAttraction();
        YamlConfig config = AttractionLoader.getConfig(attraction.getRegionId());
        config.getConfig().set("attractions." + attraction.getId() + ".name", e.getAfter());
        config.save();

        SignManager.update(attraction);
        Work.firstTask(() -> DBManager.getDatabase("public")).abortIfNull().lastTask(database -> {
            HashMap<Integer, Object> obj = new HashMap<>();
            obj.put(1, e.getAfter());
            obj.put(2, attraction.getId());

            database.execute("UPDATE attraction SET name=? WHERE id=?", obj);
        }).execute();
    }

    @EventHandler
    public void onChange(ChangeLocationEvent e) {
        Attraction attraction = e.getAttraction();
        YamlConfig config = AttractionLoader.getConfig(attraction.getRegionId());
        config.getConfig().set("attractions." + attraction.getId() + ".location", LocationSerializer.toString(e.getAfter()));
        config.save();

        MapMarker.getMarker().setAttractionMarker(attraction);
    }

    @EventHandler
    public void onRemove(RemoveAttractionEvent e) {
        if(AttractionLoader.loading)
            return;

        AttractionMenu.load();

        Attraction attraction = e.getAttraction();
        String config_path = "attractions." + attraction.getId();
        YamlConfig yc = AttractionLoader.getConfig(attraction.getRegionId());
        FileConfiguration config = yc.getConfig();
        config.set(config_path, null);
        yc.save();

        MapMarker.getMarker().removeAttractionMarker(attraction);
        Work.firstTask(() -> DBManager.getDatabase("public")).abortIfNull().lastTask(database -> {
            HashMap<Integer, Object> obj = new HashMap<>();
            obj.put(1, attraction.getId());

            database.execute("DELETE FROM attraction WHERE id=?", obj);
        }).execute();
    }

}
