package nl.iobyte.themepark.listener;

import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.attraction.Region;
import nl.iobyte.themepark.api.events.region.*;
import nl.iobyte.themepark.config.YamlConfig;
import nl.iobyte.themepark.database.DBManager;
import nl.iobyte.themepark.menu.AttractionLoader;
import nl.iobyte.themepark.menu.AttractionMenu;
import nl.iobyte.workchain.components.Work;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import java.util.HashMap;

public class RegionListener implements Listener {

    @EventHandler
    public void onCreate(CreateRegionEvent e) {
        if(AttractionLoader.loading)
            return;

        AttractionMenu.load();

        Region region = e.getRegion();
        String config_path = "regions." + region.getId();
        YamlConfig yc = ThemeParkPlugin.getInstance().getRegions();
        FileConfiguration config = yc.getConfig();
        config.set(config_path+".name", region.getName());
        config.set(config_path+".material", region.getMaterial().toString());
        config.set(config_path+".lore", region.getLore());
        config.set(config_path+".data", region.getData());
        yc.save();

        AttractionLoader.addRegion(region.getId());

        Work.firstTask(() -> DBManager.getDatabase("public")).abortIfNull().lastTask(database -> {
            HashMap<Integer, Object> obj = new HashMap<>();
            obj.put(1, region.getId());
            obj.put(2, region.getName());

            database.execute("INSERT IGNORE INTO region(id, name) VALUES (?, ?)", obj);
        }).execute();
    }

    @EventHandler
    public void onChange(ChangeNameEvent e) {
        AttractionMenu.updateRegion(e.getRegion());

        Region region = e.getRegion();
        YamlConfig config = ThemeParkPlugin.getInstance().getRegions();
        config.getConfig().set("regions." + region.getId() + ".name", e.getAfter());
        config.save();

        Work.firstTask(() -> DBManager.getDatabase("public")).abortIfNull().lastTask(database -> {
            HashMap<Integer, Object> obj = new HashMap<>();
            obj.put(1, e.getAfter());
            obj.put(2, region.getId());

            database.execute("UPDATE region SET name=? WHERE id=?", obj);
        }).execute();
    }

    @EventHandler
    public void onChange(ChangeLoreEvent e) {
        AttractionMenu.updateRegion(e.getRegion());

        Region region = e.getRegion();
        YamlConfig config = ThemeParkPlugin.getInstance().getRegions();
        config.getConfig().set("regions." + region.getId() + ".lore", e.getAfter());
        config.save();
    }

    @EventHandler
    public void onChange(ChangeMaterialEvent e) {
        AttractionMenu.updateRegion(e.getRegion());

        Region region = e.getRegion();
        YamlConfig config = ThemeParkPlugin.getInstance().getRegions();
        config.getConfig().set("regions." + region.getId() + ".material", e.getAfter().toString());
        config.save();
    }

    @EventHandler
    public void onChange(ChangeDataEvent e) {
        AttractionMenu.updateRegion(e.getRegion());

        Region region = e.getRegion();
        YamlConfig config = ThemeParkPlugin.getInstance().getRegions();
        config.getConfig().set("regions." + region.getId() + ".data", e.getAfter());
        config.save();
    }

    @EventHandler
    public void onRemove(RemoveRegionEvent e) {
        if(AttractionLoader.loading)
            return;

        AttractionMenu.load();

        Region region = e.getRegion();
        String config_path = "regions." + region.getId();
        YamlConfig yc = ThemeParkPlugin.getInstance().getRegions();
        FileConfiguration config = yc.getConfig();
        config.set(config_path, null);
        yc.save();

        Work.firstTask(() -> DBManager.getDatabase("public")).abortIfNull().lastTask(database -> {
            HashMap<Integer, Object> obj = new HashMap<>();
            obj.put(1, region.getId());

            database.execute("DELETE FROM region WHERE id=?", obj);
        }).execute();
    }

}
