package nl.iobyte.themepark.listener;

import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.API;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.attraction.manager.StatusManager;
import nl.iobyte.themepark.config.YamlConfig;
import nl.iobyte.themepark.sign.SignManager;
import nl.iobyte.themepark.sign.StatusSign;
import nl.iobyte.themepark.util.Color;
import nl.iobyte.themepark.util.MessageUtil;
import org.bukkit.Location;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.SignChangeEvent;
import org.bukkit.event.player.PlayerInteractEvent;

public class SignListener implements Listener {

    private YamlConfig config = ThemeParkPlugin.getInstance().getSettings();
    private String name = config.getConfig().getString("sign.name");
    private String title = config.getConfig().getString("sign.title");

    @EventHandler
    public void onPlace(SignChangeEvent e) {
        if(!e.getLine(0).equalsIgnoreCase(name))
            return;

        String id = e.getLine(1);
        if(id == null || id.isEmpty() || !API.isAttraction(id))
            return;

        Attraction attraction = API.getAttraction(id);
        e.setLine(0, Color.color(title));
        e.setLine(1, Color.color(attraction.getName()));
        e.setLine(2, Color.color(StatusManager.getName(attraction.getStatus())));

        Location location = e.getBlock().getLocation();
        StatusSign statusSign = new StatusSign(attraction, location);
        SignManager.addSign(statusSign);
    }

    @EventHandler
    public void onBreak(BlockBreakEvent e) {
        if(e.getBlock() == null)
            return;

        Block block = e.getBlock();
        if(!(block.getState() instanceof Sign))
            return;

        Sign sign = (Sign) block.getState();
        if(!sign.getLine(0).equals(Color.color(title)))
            return;

        Attraction attraction = API.getAttractionFromName(sign.getLine(1));
        Location location = e.getBlock().getLocation();

        StatusSign s = new StatusSign(attraction, location);
        SignManager.removeSign(s);
    }

    @EventHandler
    public void onClick(PlayerInteractEvent e) {
        if(e.getAction() != Action.RIGHT_CLICK_BLOCK)
            return;

        Block b = e.getClickedBlock();
        if(!(b.getState() instanceof Sign))
            return;

        Sign sign = (Sign) b.getState();
        if(!sign.getLine(0).equals(Color.color(title)))
            return;

        Attraction attraction = API.getAttractionFromName(sign.getLine(1));
        if(!SignManager.isSign(attraction, sign.getLocation()))
            return;

        Location location = attraction.getLocation();
        if(location == null)
            return;

        Player player = e.getPlayer();
        if(!StatusManager.canTeleport(attraction.getStatus())) {
            String message = MessageUtil.getMessage("attraction.teleport.status");
            message = message.replace("{name}", attraction.getName());
            message = message.replace("{status}", StatusManager.getName(attraction.getStatus()));
            player.sendMessage(Color.color(message));
            return;
        }

        player.teleport(location);
        String message = MessageUtil.getMessage("attraction.teleport.success");
        message = message.replace("{name}", attraction.getName());
        player.sendMessage(Color.color(message));
    }

}
