package nl.iobyte.themepark.traincarts;

import com.bergerkiller.bukkit.tc.Permission;
import com.bergerkiller.bukkit.tc.controller.MinecartMember;
import com.bergerkiller.bukkit.tc.events.SignActionEvent;
import com.bergerkiller.bukkit.tc.events.SignChangeActionEvent;
import com.bergerkiller.bukkit.tc.signactions.SignAction;
import com.bergerkiller.bukkit.tc.signactions.SignActionType;
import com.bergerkiller.bukkit.tc.utils.SignBuildOptions;
import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.API;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.events.ridecount.ChangeCountEvent;
import nl.iobyte.themepark.api.events.ridecount.PreProcessCountEvent;
import nl.iobyte.themepark.api.ridecount.AttractionCount;
import nl.iobyte.themepark.api.ridecount.CountManager;
import nl.iobyte.themepark.ridecount.RideCountAPI;
import nl.iobyte.themepark.util.Color;
import nl.iobyte.themepark.util.MessageUtil;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.atomic.AtomicBoolean;

public class TCRideCount extends SignAction {

    @Override
    public boolean match(final SignActionEvent info) {
        return info.getLine(1).equalsIgnoreCase("ridecount") && !info.getLine(2).isEmpty() && !info.getLine(3).isEmpty();
    }

    @Override
    public void execute(final SignActionEvent info) {
        if (!info.isPowered())
            return;

        if (info.isTrainSign() && info.isAction(SignActionType.REDSTONE_ON, SignActionType.GROUP_ENTER) && info.hasGroup()) {
            ArrayList<Player> players = new ArrayList<>();
            for (final MinecartMember<?> member : info.getGroup())
                players.addAll(member.getEntity().getPlayerPassengers());

            count(players, info);
            return;
        }

        if (!info.isCartSign() || !info.isAction(SignActionType.REDSTONE_ON, SignActionType.MEMBER_ENTER) || !info.hasMember()) {
            if (info.isRCSign() && info.isAction(SignActionType.REDSTONE_ON)) {
                ArrayList<Player> players = new ArrayList<>();
                for (final MinecartMember<?> member : info.getGroup())
                    players.addAll(member.getEntity().getPlayerPassengers());

                count(players, info);
            }

            return;
        }

        if (!info.getMember().getEntity().isEmpty()) {
            ArrayList<Player> players = new ArrayList<>(info.getMember().getEntity().getPlayerPassengers());
            count(players, info);
        }
    }

    @Override
    public boolean build(SignChangeActionEvent event) {
        return SignBuildOptions.create()
                .setName(event.isCartSign() ? "cart ridecount sign" : "train ridecount sign")
                .setDescription("Add a count to a player for an attraction")
                .handle(event.getPlayer());
    }

    private void count(ArrayList<Player> players, SignActionEvent info) {
        String id = info.getLine(2);
        if(!API.isAttraction(id))
            return;

        int amount = 1;
        try {
            amount = Integer.parseInt(info.getLine(3));
        } catch (Exception e) {
            e.printStackTrace();
        }

        Attraction attraction = API.getAttraction(id);
        AtomicBoolean b = new AtomicBoolean(false);
        AttractionCount count = CountManager.getCounter(attraction.getId());

        for(Player player : players) {
            PreProcessCountEvent event = new PreProcessCountEvent(player.getUniqueId(), attraction, amount);
            Bukkit.getPluginManager().callEvent(event);

            int u_amount = event.getCount();
            UUID finalUuid = player.getUniqueId();
            RideCountAPI.getCount(finalUuid, attraction).thenAccept(before -> RideCountAPI.addCount(finalUuid, attraction, u_amount).thenAccept(result -> {
                if (!result)
                    return;

                if(count == null || !count.hasCount(finalUuid)) {
                    Bukkit.getScheduler().runTask(ThemeParkPlugin.getInstance(), () -> {
                        ChangeCountEvent e = new ChangeCountEvent(attraction, count, finalUuid, before, before + u_amount);
                        Bukkit.getPluginManager().callEvent(e);
                    });
                }

                if(b.get())
                    return;

                b.set(true);
            }));
        }
    }

}
